<?php
	// Exit if accessed directly
	if( !defined( 'ABSPATH' ) ) exit;


class wpForoAttachUploadHandler extends UploadHandler {
	
    protected function set_additional_file_properties($file) {
        parent::set_additional_file_properties($file);
        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            $attach = WPF_ATTACH()->get_attach( $this->get_download_url($file->name) );
            
            $file->id = $attach['attachid'];
            $file->filename = $attach['filename'];
            $file->wpf_type = WPF_ATTACH()->tools->get_file_type($file->name);
            $file->wpf_file_fa_ico = WPF_ATTACH()->tools->get_file_fa_ico_class($file->name);
            $file->wpf_is_new = FALSE;
            
        }
    }
	
    public function delete($print_response = true) {
    	if( !empty(WPF_ATTACH()->options['disable_delete']) ) return FALSE;
    	
        $response = parent::delete(false);
        foreach ($response as $name => $deleted) {
            if ($deleted) WPF_ATTACH()->delete( $this->get_download_url($name) );
        } 
        return $this->generate_response($response, $print_response);
    }
	
	protected function handle_file_upload($uploaded_file, $name, $size, $type, $error, $index = null, $content_range = null) {
        $file = new \stdClass();
        $file_name = $this->get_file_name($uploaded_file, $name, $size, $type, $error, $index, $content_range);
        $file->size = $this->fix_integer_overflow(intval($size));
        $file->type = $type;
        $file->filename = $name;
        $file->wpf_is_new = FALSE;
        
        $ext = pathinfo($file_name, PATHINFO_EXTENSION);
        $fnm = pathinfo($file_name, PATHINFO_FILENAME);
        $fnm = str_replace(' ', '-', $fnm);
        while(strpos($fnm, '--') !== FALSE){
			$fnm = str_replace('--', '-', $fnm);
		}
        $fnm = preg_replace("/[^-a-zA-Z0-9]/", "", $fnm);
        $fnm = trim($fnm, "-");
        $fnm_empty = ( $fnm ? FALSE : TRUE );
        
        $file_name = $fnm . "." . $ext;
        
        $file->name = $file_name;
        
        $args = array(
			'filename' => $file->filename,
			'size' => $file->size,
			'mime' => $file->type
        );
        if($file->id = WPF_ATTACH()->add($args)){
        	$file->name = $file->id . ( $fnm_empty ? '' : '-' ) . $file->name;
        	
	        if ($this->validate($uploaded_file, $file, $error, $index)) {
	            $this->handle_form_data($file, $index);
	            $upload_dir = $this->get_upload_path();
	            if (!is_dir($upload_dir)) {
	                mkdir($upload_dir, $this->options['mkdir_mode'], true);
	            }
	            
	            $file_path = $this->get_upload_path($file->name);
	            
	            $append_file = $content_range && is_file($file_path) &&
	                $file->size > $this->get_file_size($file_path);
	            if ($uploaded_file && is_uploaded_file($uploaded_file)) {
	                // multipart/formdata uploads (POST method uploads)
	                if ($append_file) {
	                    file_put_contents(
	                        $file_path,
	                        fopen($uploaded_file, 'r'),
	                        FILE_APPEND
	                    );
	                } else {
	                    move_uploaded_file($uploaded_file, $file_path);
	                }
	            } else {
	                // Non-multipart uploads (PUT method support)
	                file_put_contents(
	                    $file_path,
	                    fopen('php://input', 'r'),
	                    $append_file ? FILE_APPEND : 0
	                );
	            }
	            $file_size = $this->get_file_size($file_path, $append_file);
	            if ($file_size === $file->size) {
	                $file->url = $this->get_download_url($file->name);
	                if ($this->is_valid_image_file($file_path)) {
	                    $this->handle_image_file($file_path, $file);
	                }
	            } else {
	                $file->size = $file_size;
	                if (!$content_range && $this->options['discard_aborted_uploads']) {
	                    unlink($file_path);
	                    $file->error = $this->get_error_message('abort');
	                }
	            }
				
	            
	            $this->set_additional_file_properties($file);
	        }
	        
	    }
        
        if (empty($file->error) && $file->id){
        	$file->wpf_is_new = TRUE;
        	WPF_ATTACH()->edit( array('fileurl' => $file->url), $file->id );
        }elseif($file->id){
			WPF_ATTACH()->delete($file->id);
		}
        
        $file->wpf_type = WPF_ATTACH()->tools->get_file_type($file->name);
        $file->wpf_file_fa_ico = WPF_ATTACH()->tools->get_file_fa_ico_class($file->name);
        
        return $file;
    }
    
    private function sortByCreationTime($file_1, $file_2)
	{
		$upload_dir = $this->get_upload_path();
	    $file_1 = filectime($upload_dir . $file_1);
	    $file_2 = filectime($upload_dir . $file_2);
	    if($file_1 == $file_2)
	    {
	        return 0;
	    }
	    return $file_1 < $file_2 ? 1 : -1;
	}

    protected function get_file_objects($iteration_method = 'get_file_object') {
        $upload_dir = $this->get_upload_path();
        if (!is_dir($upload_dir)) {
            return array();
        }
        
        $files = scandir($upload_dir);
		usort($files, array(&$this,'sortByCreationTime') );
        
        return array_values(array_filter(array_map(
            array($this, $iteration_method),
            $files
        )));
    }
    
}