<?php
	// Exit if accessed directly
	if( !defined( 'ABSPATH' ) ) exit;

function wpforoattach_init(){
	if( function_exists('WPF') ){
		$GLOBALS['wpforoattach'] = WPF_ATTACH();
	}else{
		add_action( 'admin_notices', 'wpforoattach_admin_notice__error' );
	}
}
add_action('plugins_loaded', 'wpforoattach_init');

register_activation_hook(WPFOROATTACH_BASENAME, 'do_wpforoattach_activation');
function do_wpforoattach_activation($network_wide){
	if ( is_multisite() && $network_wide ) { 
        global $wpdb;
		
        $old_blogid = $wpdb->blogid;
        $blogids = $wpdb->get_col("SELECT `blog_id` FROM {$wpdb->blogs}");
        foreach ($blogids as $blogid){
            switch_to_blog($blogid);
            wpforoattach_activation();
        }
        switch_to_blog($old_blogid);
    }else{
        wpforoattach_activation();
    }
}

function wpforoattach_activation(){
	global $wpdb;
	
	$charset_collate = '';
	if ( ! empty($wpdb->charset) ) $charset_collate = "DEFAULT CHARACTER SET " . $wpdb->charset;
	if ( ! empty($wpdb->collate) ) $charset_collate .= " COLLATE " . $wpdb->collate;
	$engine = version_compare($wpdb->db_version(), '5.6.4', '>=') ? 'InnoDB' : 'MyISAM';
	
	$sql = "CREATE TABLE IF NOT EXISTS `".$wpdb->prefix."wpforo_attachments`(  
	  `attachid` INT NOT NULL AUTO_INCREMENT,
	  `userid` INT NOT NULL,
	  `filename` varchar(255) NOT NULL,
	  `fileurl` VARCHAR(255) NOT NULL,
	  `size` INT NOT NULL DEFAULT 0,
	  `mime` varchar(255) NOT NULL,
	  `posts` INT NOT NULL DEFAULT 0,
	  PRIMARY KEY (`attachid`)
	)ENGINE=$engine $charset_collate";
	
	@$wpdb->query( $sql );

    $sql = "SHOW COLUMNS FROM `".$wpdb->prefix."wpforo_attachments` LIKE 'posts'";
    if( !$wpdb->get_row($sql, ARRAY_A) ){
        $sql = "ALTER TABLE `".$wpdb->prefix."wpforo_attachments` ADD COLUMN `posts` INT UNSIGNED NOT NULL DEFAULT 0";
        @$wpdb->query( $sql );
    }
	
	// SET Attachments Options //////////////////////////////////////////////////////////////////
	$mime_types = get_allowed_mime_types();
	$mime_types = array_flip($mime_types);
	if(!empty($mime_types)){
		$allowed_types = implode('|', $mime_types);
	}else{
		$allowed_types = 'gif|jpeg|jpg|png|mp4|mp3';
	}
	
	$umf = (int) ini_get('upload_max_filesize');
	$pms = (int) ini_get('post_max_size');
	$min = min($umf,$pms);
	
	$wpforo_attachment = array(
		'accepted_file_types' => $allowed_types,
		'maximum_file_size' => ( $min ? $min : 50 ),
		'thumbnail_width' => 400,
		'thumbnail_height' => 300,
		'boxed' => 0,
		'lightbox' => 1,
		'disable_delete' => 1
	);
	add_option( 'wpforo_attach_options', $wpforo_attachment );
	
	$wpforo_phrases = array(
		'attach files' => __('Attach Files', 'wpforo_attach'),
		'attached file' => __('Attached File', 'wpforo_attach'),
		'add files...' => __('Add files...', 'wpforo_attach'),
		'start upload' => __('Start upload', 'wpforo_attach'),
		'insert into post' => __('Insert into post', 'wpforo_attach'),
		'start' => __('Start', 'wpforo_attach'),
		'cancel' => __('Cancel', 'wpforo_attach'),
		'delete' => __('delete', 'wpforo_attach'),
		'close' => __('Close', 'wpforo_attach'),
		'select all' => __('select all', 'wpforo_attach'),
		'processing...' => __('Processing...', 'wpforo_attach'),
		'choose a file or drag it here' => __('Choose a file or drag it here', 'wpforo_attach'),
		'please select a file(s) using right checkbox(es) to insert in post' => __('Please select a file(s) using right checkbox(es) to insert in post', 'wpforo_attach'),
		'error: file is too big. allowed size is: %s' => __('Error: File is too big. Allowed size is: %s', 'wpforo_attach'),
		'error: file is too big. server_upload_max_filesize is: %s' => __('Error: File is too big. server_upload_max_filesize is: %s', 'wpforo_attach'),
		'error: file is too big. server_post_max_size is: %s' => __('Error: File is too big. server_post_max_size is: %s', 'wpforo_attach'),
		'error: filetype not allowed' => __('Error: Filetype not allowed', 'wpforo_attach'),
		'the uploaded file exceeds the upload_max_filesize directive in php.ini' => __('The uploaded file exceeds the upload_max_filesize directive in php.ini', 'wpforo_attach'),
		'the uploaded file exceeds the max_file_size directive that was specified in the html form' => __('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form', 'wpforo_attach'),
		'the uploaded file was only partially uploaded' => __('The uploaded file was only partially uploaded', 'wpforo_attach'),
		'no file was uploaded' => __('No file was uploaded', 'wpforo_attach'),
		'missing a temporary folder' => __('Missing a temporary folder', 'wpforo_attach'),
		'failed to write file to disk' => __('Failed to write file to disk', 'wpforo_attach'),
		'a php extension stopped the file upload' => __('A PHP extension stopped the file upload', 'wpforo_attach'),
		'the uploaded file exceeds the post_max_size directive in php.ini' => __('The uploaded file exceeds the post_max_size directive in php.ini', 'wpforo_attach'),
		'file is too big' => __('File is too big', 'wpforo_attach'),
		'file is too small' => __('File is too small', 'wpforo_attach'),
		'filetype not allowed' => __('Filetype not allowed', 'wpforo_attach'),
		'maximum number of files exceeded' => __('Maximum number of files exceeded', 'wpforo_attach'),
		'image exceeds maximum width' => __('Image exceeds maximum width', 'wpforo_attach'),
		'image requires a minimum width' => __('Image requires a minimum width', 'wpforo_attach'),
		'image exceeds maximum height' => __('Image exceeds maximum height', 'wpforo_attach'),
		'image requires a minimum height' => __('Image requires a minimum height', 'wpforo_attach'),
		'file upload aborted' => __('File upload aborted', 'wpforo_attach'),
		'failed to resize image' => __('Failed to resize image', 'wpforo_attach')
	);
	add_option( 'wpforo_attach_phrases', $wpforo_phrases );
	
	$upload_array = wp_upload_dir();
	$attachs_upload_dir=$upload_array['basedir'].'/wpforo/attachments/';
	/* Create the directory where attachments upload the files */
	if (!is_dir($attachs_upload_dir)) {
		wp_mkdir_p($attachs_upload_dir);
	}
}

function wpforoattach_admin_notice__error() {
	$class = 'notice notice-error';
	$message = __( 'IMPORTANT: wpForo Attachments plugin is a wpForo extension, please install wpForo plugin.', 'wpforoattach' );
	printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message ); 
}

function wpforo_attach_phrase( $phrase_key, $echo = true ){
	$phrase_key = strtolower($phrase_key);
	if( isset(WPF_ATTACH()->phrases[$phrase_key]) ) {
		$phrase = stripslashes(WPF_ATTACH()->phrases[$phrase_key]);
	}else{
		$phrase = ucfirst(str_replace('_', ' ', $phrase_key));
	}
	if($echo) echo $phrase; else return $phrase;
}

function wpforo_attach_server_extension_status(){
    ?>
    <tr class="wpf-dw-tr">
        <td class="wpf-dw-td">GD and Image Functions</td>
        <td class="wpf-dw-td-value"><?php echo function_exists('getimagesize') && function_exists('imagecreatetruecolor') && function_exists('exif_read_data') ? '<span class="wpf-green">' . __('Available', 'wpforo') . '</span>' : '<span class="wpf-red">' . __('Not available', 'wpforo') . '</span> | <a href="http://php.net/manual/en/ref.image.php" target="_blank">more info&raquo;</a>'; ?> </td>
    </tr>
    <tr class="wpf-dw-tr">
        <td class="wpf-dw-td">PHP imagick extension</td>
        <td class="wpf-dw-td-value"><?php echo extension_loaded('imagick') ? '<span class="wpf-green">' . __('Available', 'wpforo') . '</span>' : '<span class="wpf-red">' . __('Not available', 'wpforo') . '</span> ( <span style="font-size: smaller; color: gray;">'.__('Not required if "GD and Image Functions" are	Available', 'wpforo_attach').'</span> ) | <a href="http://php.net/manual/en/imagick.installation.php" target="_blank">more info&raquo;</a>'; ?></td>
    </tr>
    <?php
}
add_action('wpforo_dashboard_widget_server', 'wpforo_attach_server_extension_status');