<?php 
/*
* Plugin Name: wpForo Advanced Attachments
* Plugin URI: https://wpForo.com
* Description: Advanced file attachment system for forum topics and posts. AJAX powered media uploading and displaying system with user specific media library.
* Author: gVectors Team (A. Chakhoyan and R. Hovhannisyan)
* Author URI: https://gvectors.com/
* Version: 1.2.2
* Text Domain: wpforo_attach
* Domain Path: /languages
*/

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

define('WPFOROATTACH_DIR', rtrim( str_replace( '//', '/', dirname(__FILE__)), '/'));
define('WPFOROATTACH_URL', rtrim( plugins_url( '', __FILE__ ), '/'));
define('WPFOROATTACH_FOLDER', rtrim( plugin_basename(dirname(__FILE__)), '/'));
define('WPFOROATTACH_BASENAME', plugin_basename( __FILE__ ));

function wpforo_attach_load_plugin_textdomain() { load_plugin_textdomain( 'wpforo_attach', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' ); }
add_action( 'plugins_loaded', 'wpforo_attach_load_plugin_textdomain' );


if( !class_exists('wpForoAttachments') ){
    class wpForoAttachments{
        private static $_instance = NULL;

        public $tools;
        public $list_table;
        public $options;
        public $phrases;

        public static function instance(){
            if( is_null(self::$_instance) ) self::$_instance = new self();
            return self::$_instance;
        }

        private function __construct(){
            $this->includes();

            $this->tools = new wpForoAttachmentsTools();

            add_action('init', array($this, 'init_options'));
            add_action('init', array($this, 'init_hooks'));

            new GVT_API_Manager(__FILE__, 'wpforo-settings&tab=plugins&subtab=attachments','wpforo_settings_page_top');
        }

        private function includes(){
            require_once(WPFOROATTACH_DIR . '/includes/class.wpForoAttachmentsTools.php');
            require_once(WPFOROATTACH_DIR . '/includes/class.wpForoAttachmentsListTable.php');
            require_once(WPFOROATTACH_DIR . "/includes/gvt-api-manager.php");
        }

        public function init_options(){
            $this->options = get_option('wpforo_attach_options');
            if( !isset($this->options['disable_delete']) ) $this->options['disable_delete'] = 1;
            //START hardcode if admin or moderator
                if( ( !isset($_GET['page']) || $_GET['page'] != 'wpforo-settings' ) && current_user_can('administrator') /*WPF()->current_user_groupid == 1 || WPF()->current_user_groupid == 2*/ )
                    $this->options['disable_delete'] = 0;
            //END hardcode if admin or moderator
            //START hardcode restrict file types for current NEW user
                $this->options['accepted_file_types'] = $this->tools->strip_not_allowed_filetypes($this->options['accepted_file_types']);
            //END hardcode restrict file types for current NEW user
            $this->options['server_upload_max_filesize'] = (int) ini_get('upload_max_filesize');
            $this->options['server_post_max_size'] = (int) ini_get('post_max_size');
            $this->options['server_maxs_min'] = min($this->options['server_upload_max_filesize'], $this->options['server_post_max_size']);
            $this->phrases = get_option('wpforo_attach_phrases');
        }

        public function init_list_table(){
            if( empty($_GET['page']) || $_GET['page'] != 'wpforo-advanced-attachments' ) return;
            $this->list_table = new wpForoAttachmentsListTable();
            $this->list_table->process_bulk_action();
        }

        public function init_hooks(){
            add_filter('wpforo_get_statistic_array_filter',  array(&$this, 'add_statistic'));
            add_filter('wpforo_plugins_tabs_array_filter', array(&$this, 'add_options_tab'));
            add_filter('wpforo_plugins_option_files_array_filter', array(&$this, 'add_options_file_path'));
            add_action('admin_menu', array($this, 'add_admin_menus'), 40);
            add_action('wp_enqueue_scripts', array($this, 'css_js_enqueue'));
            add_action('admin_enqueue_scripts', array($this, 'lightbox_css_js_enqueue'));
            add_action('wpforo_actions', array($this, 'do_actions'), 10, 0);
            add_action('wpforo_topic_form_buttons_hook', array($this, 'add_frontend_form_button'), 9, 0);
            add_action('wpforo_reply_form_buttons_hook', array($this, 'add_frontend_form_button'), 9, 0);
            add_action('wpforo_topic_info_end', array($this, 'add_attach_icon'), 1, 1);
            add_action('wp_footer', array($this, 'add_footer_html'), 99, 0);
            add_action('wpforo_after_add_topic', array($this, 'after_add_edit_post'));
            add_action('wpforo_after_edit_topic', array($this, 'after_add_edit_post'));
            add_action('wpforo_after_add_post', array($this, 'after_add_edit_post'));
            add_action('wpforo_after_edit_post', array($this, 'after_add_edit_post'));
            add_action('wpforo_after_delete_post', array($this, 'after_delete_post'));
            add_action('admin_init', array($this, 'init_list_table'));
            add_filter('wpforo_dynamic_css_filter', array($this, 'add_dynamic_css'), 10, 2);
        }

        public function add( $args ){
            $default = array(
                'userid' => NULL,
                'filename' => '',
                'fileurl' => '#',
                'size' => NULL,
                'mime' => '',
            );
            $args = wpforo_parse_args( $args, $default );
            extract($args, EXTR_OVERWRITE);
            if( !$filename || !$fileurl ) return FALSE;

            if( !$userid ) $userid = WPF()->current_userid;

            if( WPF()->db->insert(
                    WPF()->db->prefix . 'wpforo_attachments',
                    array(
                        'userid' => $userid,
                        'filename' => $filename,
                        'fileurl' => $fileurl,
                        'size' => $size,
                        'mime' => $mime
                    ),
                    array('%d','%s','%s','%d','%s')
                )
            ){ return WPF()->db->insert_id; }

            return FALSE;
        }

        public function edit( $args, $attachid ){
            if( empty($args) || !($attachid = wpforo_bigintval($attachid)) ) return FALSE;
            $default = array(
                'userid' => NULL,
                'filename' => '',
                'fileurl' => '',
                'size' => NULL,
                'mime' => '',
                'posts' => NULL
            );
            $args = wpforo_parse_args( $args, $default );

            $sets = array_filter($args);
            $format = array();
            foreach( $sets as $set ){
                if( $set === NULL ){
                    $format[] = '%d';
                }else{
                    $format[] = '%s';
                }
            }

            if( !empty($set) ){
                if( FALSE !== WPF()->db->update(
                        WPF()->db->prefix . 'wpforo_attachments',
                        $sets,
                        array( 'attachid' => $attachid ),
                        $format,
                        array('%d')
                    )
                ){ return $attachid; }
            }

            return FALSE;
        }

        public function delete($arg){
            if(empty($arg)) return FALSE;

            $where = array();
            $format = array();
            if( is_numeric($arg) ){
                $where['attachid'] = wpforo_bigintval($arg);
                $format[] = '%d';
            }else{
                $where['fileurl'] = trim($arg);
                $format[] = '%s';
            }

            if(FALSE !== WPF()->db->delete(
                WPF()->db->prefix . 'wpforo_attachments',
                $where,
                $format
            )){ return $arg; }

            return FALSE;
        }

        public function get_attach($arg){
            if(empty($arg)) return FALSE;

            $where = '';
            $format = array();
            if( is_numeric($arg) ){
                $where = "`attachid` = %d";
                $arg = wpforo_bigintval($arg);
            }else{
                $where = "`fileurl` = %s";
                $arg = trim($arg);
            }
            $sql = "SELECT * FROM `".WPF()->db->prefix."wpforo_attachments` WHERE $where";
            return WPF()->db->get_row( WPF()->db->prepare($sql, $arg), ARRAY_A );
        }

        public function get_attachs( $args = array(), &$items_count = 0 ){
            $default = array(
                'include'   => array(),      // array( 2, 10, 25 ) or '2,10,25'
                'exclude'   => array(),      // array( 2, 10, 25 ) or '2,10,25'
                'userid'    => NULL,        // user id in DB
                'mime'      => NULL,
                'orderby'   => 'attachid DESC',   // attachid|size|posts
                'order'     => '',       // ASC DESC
                'offset'    => NULL,        // this use when you give row_count
                'row_count' => NULL        // 4 or 1 ...
            );
            if( empty($args['orderby']) ) $args['order'] = '';

            $args = wpforo_parse_args( $args, $default );
            extract($args);

            $include = wpforo_parse_args( $include );
            $exclude = wpforo_parse_args( $exclude );

            $sql = "SELECT * FROM `" . WPF()->db->prefix . "wpforo_attachments`";
            $wheres = array();

            if($include) $wheres[] = "`attachid` IN(" . implode(',', array_map('wpforo_bigintval', $include)) . ")";
            if($exclude) $wheres[] = "`attachid` NOT IN(" . implode(',', array_map('wpforo_bigintval', $exclude)) . ")";
            if($userid)  $wheres[] = "`userid` = " . wpforo_bigintval($userid);
            if($mime)    $wheres[] = "`mime` LIKE '" . trim($mime) . "'";

            if($wheres) $sql .= " WHERE " . implode( " AND ", $wheres );

            $item_count_sql = preg_replace('#SELECT.+?FROM#isu', 'SELECT count(*) FROM', $sql);
            if( $item_count_sql ) $items_count = WPF()->db->get_var($item_count_sql);

            $sql .= esc_sql(" ORDER BY $orderby " . $order);

            if($row_count) $sql .= esc_sql(" LIMIT ".intval($offset).",".intval($row_count));

            return WPF()->db->get_results($sql, ARRAY_A);
        }

        public function search($needle, $fields = array(), $limit = NULL){
            $attachids = array();
            if(!$needle) return $attachids;

            $needle = sanitize_text_field($needle);
            if(empty($fields)){
                $fields = array(
                    'filename',
                    'fileurl',
                    'mime'
                );
            }

            $sql = "SELECT `attachid` FROM `".WPF()->db->prefix."wpforo_attachments`";
            $wheres = array();

            foreach($fields as $field){
                $field = sanitize_text_field($field);
                $wheres[] = "`".esc_sql($field)."` LIKE '%" . esc_sql($needle) ."%'";
            }

            if(!empty($wheres)){
                $sql .= " WHERE " . implode($wheres, " OR ");
                if( $limit ) $sql .= " LIMIT " . intval($limit);

                $attachids = WPF()->db->get_col($sql);
            }

            return $attachids;
        }

        public function search_possible_spams(){
            $attachids = array();

            $spam_file_phrases = array(
                0 => array( 'watch', 'movie'),
                1 => array( 'download', 'free')
            );

            $ext_risk = array('pdf', 'doc', 'docx', 'txt', 'htm', 'html', 'rtf', 'xml', 'xls', 'xlsx', 'php', 'cgi', 'php', 'cgi', 'exe');

            $sql = "SELECT `attachid` FROM `".WPF()->db->prefix."wpforo_attachments`";
            $wheres = array();

            foreach($spam_file_phrases as $phrases) {
                foreach ($phrases as $phrase) {
                    $phrase = sanitize_text_field($phrase);
                    $wheres[] = "`fileurl` LIKE '%" . esc_sql($phrase) ."%'";
                }
            }
            if($wheres) $sql .= " WHERE (". implode($wheres, " OR ") .")";

            $wheres = array();
            foreach ($ext_risk as $ext){
                $ext = sanitize_text_field($ext);
                $wheres[] = "`fileurl` LIKE '%" . esc_sql($ext) ."'";
            }
            if($wheres){
                $sql .= " AND (". implode($wheres, " OR ") .")";

                $attachids = WPF()->db->get_col($sql);
            }

            return $attachids;
        }

        public function get_distinct_userids(){
            return WPF()->db->get_col("SELECT DISTINCT `userid` FROM `".WPF()->db->prefix."wpforo_attachments`");
        }

        public function get_distinct_mimes(){
            return WPF()->db->get_col("SELECT DISTINCT `mime` FROM `".WPF()->db->prefix."wpforo_attachments`");
        }

        public function get_post_count_by_attach($attachid){
            if( empty($attachid) ) return false;
            $sql = "SELECT COUNT(`postid`) FROM `".WPF()->db->prefix."wpforo_posts` WHERE `body` REGEXP '\\\[attach\\\]([^\]]*,)?%d(,[^\[]*)?\\\[/attach\\\]'";
            return WPF()->db->get_var( WPF()->db->prepare($sql, $attachid) );
        }

        public function is_attach_have_posts($attachid){
            if( $this->get_post_count_by_attach($attachid) ) return true;
            return false;
        }

        public function remove_file($attachid){
            if( $attach = $this->get_attach($attachid) ) {
                $upload_dir = wp_upload_dir();
                $attachs_dir = $upload_dir['basedir']."/wpforo/attachments";
                $fileurl_dirname = basename(dirname($attach['fileurl']));
                $fileurl_basename = basename($attach['fileurl']);

                $full_file_addrss = $attachs_dir . "/" . $fileurl_dirname . '/' . $fileurl_basename;
                if( file_exists($full_file_addrss) ){
                    $d1 = unlink($full_file_addrss);
                }else{
                    $d1 = true;
                }

                $full_thumbfile_addrss = $attachs_dir . "/" . $fileurl_dirname . '/thumbnail/' . $fileurl_basename;
                if( file_exists($full_thumbfile_addrss) ){
                    $d2 = unlink($full_thumbfile_addrss);
                }else{
                    $d2 = true;
                }

                if ($d1 && $d2) return true;
            }
            return false;
        }

        public function detect_post_body_attachids($body){
            $attachids = array();
            if( preg_match_all('#\[attach\]([^\[\]]+?)\[\/attach\]#isu', $body, $matches, PREG_SET_ORDER) ){
                foreach ($matches as $match) {
                    $expld = explode(',', $match[1]);
                    $attachids = array_merge($attachids, $expld);
                }
            }

            return $attachids;
        }

        public function rebuild_post_count($attachid){
            if( empty($attachid) ) return false;
            if( !$post_count = $this->get_post_count_by_attach($attachid) ) $post_count = 0;
            $this->edit( array('posts' => $post_count), $attachid );
            return $post_count;
        }

        public function after_add_edit_post($post){
            if( empty($post['body']) ) return;
            if( $attachids = $this->detect_post_body_attachids($post['body']) ){
                foreach ($attachids as $attachid){
                    $this->rebuild_post_count($attachid);
                }
            }
        }

        public function after_delete_post($post){
            if( empty($post['body']) ) return;
            if( $attachids = $this->detect_post_body_attachids($post['body']) ){
                foreach( $attachids as $attachid ){
                    if( !$this->is_attach_have_posts($attachid) ){
                        if( $this->remove_file($attachid) ) $this->delete($attachid);
                    }
                }
            }
        }

        public function get_count(){
            return WPF()->db->get_var( "SELECT COUNT(`attachid`) FROM `".WPF()->db->prefix."wpforo_attachments`" );
        }

        public function get_sizes(){
            return WPF()->db->get_var( "SELECT sum(`size`) FROM ".WPF()->db->prefix."wpforo_attachments" );
        }

        public function add_admin_menus(){
            add_submenu_page('upload.php', __('Forum Attachments', 'wpforo'), __('Forum Attachments', 'wpforo'), 'activate_plugins', 'wpforo-advanced-attachments', array($this, 'medialib_page'));
            add_submenu_page('wpforo-community', __('Attachments', 'wpforo'), __('Attachments', 'wpforo'), 'activate_plugins', 'wpforo-advanced-attachments', array($this, 'medialib_page'));
        }

        public function medialib_page(){
            $this->list_table->prepare_items();

            ?>
            <div id="wpf-admin-wrap" class="wrap">
                <?php wpforo_screen_option() ?>

                <div id="icon-users" class="icon32"><br></div>
                <h2 style="padding:30px 0px 0px 0px;line-height: 20px; margin-bottom:15px;"><?php _e('Media Library - wpForo Advanced Attachments', 'wpforo_attach'); ?></h2>

                <?php WPF()->notice->show(false); ?>
                <br><hr>

                <!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
                <form id="wpf-attach-lib" method="get">
                    <!-- For plugins, we also need to ensure that the form posts back to our current page -->
                    <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />

                    <?php $this->list_table->users_dropdown() ?>
                    <?php $this->list_table->mimes_dropdown() ?>
                    <input type="submit" value="Filter" class="button button-large" />

                    <!-- Now we can render the completed list table -->
                    <?php $this->list_table->search_box('Search Attachs', 'wpf-attach-search') ?>
                    <hr>
                    <a href="<?php echo admin_url('admin.php?page=wpforo-advanced-attachments') ?>">All Attachments(<?php echo $this->get_count() ?>)</a>&nbsp;
                    <?php $possible_spam_count = count( $this->search_possible_spams() ) ?>
                    <a href="<?php echo admin_url('admin.php?page=wpforo-advanced-attachments&filter=possible-spam') ?>" style="color: <?php echo ( $possible_spam_count ? 'red' : 'gray' ) ?>;">
                        Possible Spam Attachments(<?php echo $possible_spam_count ?>)
                    </a>
                    <?php if ( $possible_spam_count && isset($_REQUEST['filter']) && $_REQUEST['filter'] == 'possible-spam' ) : ?>
                        &nbsp;&nbsp;
                        <a onclick="return confirm('<?php wpforo_phrase("Are you sure you whant to empty all possible spam attachments?") ?>');"
                           href="<?php echo wp_nonce_url( admin_url('admin.php?page=wpforo-advanced-attachments&filter=possible-spam&spam=empty'), 'bulk-spam-empty' ) ?>">
                            <button type="button">Empty Possible Spams</button>
                        </a>
                    <?php endif ?>
                    <?php $this->list_table->display() ?>
                </form>

            </div>
            <?php if( $this->options['lightbox'] ): ?>
                <!-- The blueimp Gallery widget for content images -->
                <div style="display: none;" id="wpf-content-blueimp-gallery" class="blueimp-gallery blueimp-gallery-controls">
                    <div class="slides"></div><h3 class="title"></h3>
                    <a class="prev">‹</a><a class="next">›</a><a class="close">×</a><a class="play-pause"></a><ol class="indicator"></ol>
                </div>
                <?php
            endif;
        }

        public function add_statistic($counts){
            $counts['attachments'] = $this->get_count();
            $counts['attachment_sizes'] = $this->get_sizes();
            return $counts;
        }

        public function add_options_tab($tabs){
            $tabs['attachments'] = 'Attachments';
            return $tabs;
        }

        public function add_options_file_path($option_files){
            $option_files['attachments'] = WPFOROATTACH_DIR . "/includes/options.php";
            return $option_files;
        }

        public function add_frontend_form_button(){
            if( !WPF()->perm->forum_can('a') || !WPF()->perm->can_attach() ) return;
            echo ' &nbsp;&nbsp;|&nbsp;&nbsp; <a id="wpf_attach_button" style="cursor: pointer; font-size: 14px;"><i class="fas fa-paperclip" style="font-size: 18px;"></i>&nbsp;&nbsp;'. wpforo_attach_phrase('Attach Files', false) .'</a>';
        }

        public function add_attach_icon( $args = array() ){
            if( $args['has_attach'] ) echo ' <i class="fas fa-paperclip" aria-hidden="true" title="'. wpforo_attach_phrase('Attach Files', false) .'"></i> ';
        }

        public function add_footer_html(){
            if( WPF()->current_object['template'] != 'post'
                    && WPF()->current_object['template'] != 'topic'
                        && WPF()->current_object['template'] != 'messages') return false;

            if(is_user_logged_in()) : ?>
                <div id="wpfa_dialog_wrap">
                    <div id="wpfa_dialog">
                        <div id="wpfa_dialog_header">
                            <i id="wpfa_dialog_close" class="fas fa-window-close fa-2x"></i>
                        </div>
                        <div id="wpfa_dialog_body">
                            <form id="wpfa_fileupload" action="<?php echo admin_url('admin-ajax.php') ?>" method="POST" enctype="multipart/form-data" >
                                <input type="hidden" name="action" value="wpforoattach_load_ajax_function" />
                                <div id="wpfa_dialog_action_buttons" class="wpfa-fileupload-buttonbar">
                                    <div class="wpfa-fileupload-buttons">
                                        <div class="wpfa-buttons-group">
                                            <label title="<?php wpforo_attach_phrase('Choose a file or Drag it here') ?>" for="wpfa-file-input" class="wpfa-button">
                                                <span class="wpfa-button-icon"><i class="fas fa-folder-open"></i></span>
                                                <span title="<?php wpforo_attach_phrase('Choose a file or Drag it here') ?>" class="wpfa-button-text"><?php wpforo_attach_phrase('Attach Files') ?></span>
                                                <input title="<?php wpforo_attach_phrase('Choose a file or Drag it here') ?>" id="wpfa-file-input" type="file" name="files[]" multiple class="add" style="cursor: pointer;">
                                            </label>
                                            <button type="button" class="wpfa-start wpfa-button wpfa-button-start">
                                                <span class="wpfa-button-icon"><i class="fas fa-upload"></i></span>
                                                <span class="wpfa-button-text"><?php wpforo_attach_phrase('Start upload') ?></span>
                                            </button>
                                            <button id="wpf_attach_do" type="button" class="wpfa-button wpfa-button-add-to-post">
                                                <span class="wpfa-button-icon"><i class="fas fa-arrow-circle-right"></i></span>
                                                <span class="wpfa-button-text"><?php wpforo_attach_phrase('Insert into post') ?></span>
                                            </button>
                                            <button type="reset" class="wpfa-cancel wpfa-button wpfa-button-cancel wpfa-show-for-large">
                                                <span class="wpfa-button-icon"><i class="fas fa-ban"></i></span>
                                                <span class="wpfa-button-text"><?php wpforo_attach_phrase('Cancel') ?></span>
                                            </button>
                                        </div>

                                        <?php if( empty(WPF_ATTACH()->options['disable_delete']) ) : ?>
                                            <button type="button" class="wpfa-delete wpfa-button wpfa-button-delete wpfa-show-for-large">
                                                <span class="wpfa-button-icon"><i class="fas fa-trash-alt"></i></span>
                                                <span class="wpfa-button-text"><?php wpforo_attach_phrase('Delete') ?></span>
                                            </button>
                                        <?php endif; ?>
                                        <label class="wpfa-checkbox-select-all wpfa-show-for-large" title="<?php echo esc_attr(wpforo_attach_phrase('select all', false)) ?>"><input type="checkbox" class="wpfa-toggle"></label>
                                    </div>
                                    <div class="wpfa-fileupload-progress wpforo-fade" style="display:none;max-width:500px;margin-top:2px;">
                                        <div class="wpfa-progress" role="progressbar" aria-valuemin="0" aria-valuemax="100"></div>
                                        <div class="wpfa-progress-extended"></div>
                                    </div>
                                </div>
                                <div id="wpfa_dialog_items" role="presentation">
                                    <i style="margin-top: 25%; font-size: 5em;" class="fas fa-spinner fa-spin fa-5x"></i>
                                </div>
                            </form>
                        </div>
                    </div>
                </div>
            <?php endif ?>
            <?php if( $this->options['lightbox'] ): ?>
                <!-- The blueimp Gallery widget for dialog -->
                <div id="blueimp-gallery" class="blueimp-gallery blueimp-gallery-controls" data-filter=":even">
                    <div class="slides"></div><h3 class="title"></h3>
                    <a class="prev">‹</a><a class="next">›</a><a class="close">×</a><a class="play-pause"></a><ol class="indicator"></ol>
                </div>
                <!-- The blueimp Gallery widget for content images -->
                <div id="wpf-content-blueimp-gallery" class="blueimp-gallery blueimp-gallery-controls">
                    <div class="slides"></div><h3 class="title"></h3>
                    <a class="prev">‹</a><a class="next">›</a><a class="close">×</a><a class="play-pause"></a><ol class="indicator"></ol>
                </div>
            <?php endif ?>
            <?php
        }

        public function do_actions(){
            if( isset($_POST['wpforo_attach_options']) ){
                $accepted_file_types = '';
                $explds = array_filter( explode( '|', trim($_POST['wpforo_attach_options']['accepted_file_types'], '|') ) );
                if( !empty($explds) ) $accepted_file_types = implode( '|', array_map('trim', $explds) );
                $_POST['wpforo_attach_options']['accepted_file_types'] = $accepted_file_types;

                if( $_POST['wpforo_attach_options']['maximum_file_size'] > $this->options['server_maxs_min'] ) $_POST['wpforo_attach_options']['maximum_file_size'] = $this->options['server_maxs_min'];

                update_option("wpforo_attach_options", $_POST['wpforo_attach_options']);
                update_option("wpforo_attach_phrases", $_POST['wpforo_attach_phrases']);

                WPF()->notice->add('Options Saved', 'success');
                wp_redirect( wpforo_get_request_uri() );
                exit();
            }
        }

        public  function lightbox_css_js_enqueue(){
            $wpf_stylepath = WPFOROATTACH_URL . '/wpf-third-party/file-uploader/css/';
            $wpf_scriptpath = WPFOROATTACH_URL . '/wpf-third-party/file-uploader/js/';
            if( $this->options['lightbox'] ){
                wp_register_style( 'blueimp-gallery-style', $wpf_stylepath . 'blueimp-gallery.min.css', false, false );
                wp_enqueue_style('blueimp-gallery-style');
                wp_register_script('jquery-blueimp-gallery-script', $wpf_scriptpath . 'jquery.blueimp-gallery.min.js', array('jquery'), false, false );
                wp_enqueue_script('jquery-blueimp-gallery-script');
            }
        }

        public function css_js_enqueue(){
            if( WPF()->current_object['template'] != 'post' && WPF()->current_object['template'] != 'topic' && WPF()->current_object['template'] != 'messages') return;
            $wpf_scriptpath = WPFOROATTACH_URL . '/wpf-third-party/file-uploader/js/';

            $this->lightbox_css_js_enqueue();

            if(is_user_logged_in()){
                wp_register_script('jquery-ui-script', $wpf_scriptpath . 'jquery-ui.min.js', array('jquery'), false, false );
                wp_enqueue_script('jquery-ui-script');
                wp_register_script('wpf_attach', WPFOROATTACH_URL . '/assets/js/attach.js', false, false );
                wp_enqueue_script('wpf_attach');
                wp_localize_script('wpf_attach', 'wpf_front_ajax_obj', array('url' => admin_url('admin-ajax.php')));
            }
            wp_register_style( 'wpfa-style', WPFOROATTACH_URL . '/assets/css/style.css', false, false );
            wp_enqueue_style('wpfa-style');
        }

        public function css_js_ajaxload(){
            $wpf_scriptpath = WPFOROATTACH_URL . '/wpf-third-party/file-uploader/js/';
            $wpfa_js_path = WPFOROATTACH_URL . '/assets/js/';?>
            <?php if(is_user_logged_in()) : ?>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>load-image.all.min.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>tmpl.min.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>canvas-to-blob.min.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.iframe-transport.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload-process.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload-image.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload-audio.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload-video.js"></script>
                <script type="text/javascript" src="<?php echo $wpf_scriptpath ?>jquery.fileupload-validate.js"></script>

                <script type="text/javascript" src="<?php echo $wpfa_js_path   ?>wpfa-jquery.fileupload-ui.js"></script>
                <script type="text/javascript" src="<?php echo $wpfa_js_path   ?>wpfa-jquery.fileupload-jquery-ui.js"></script>
            <?php endif;
        }

        public function add_dynamic_css($css, $COLORS){
            extract($COLORS);
            $css .= '
                #wpfa_dialog #wpfa_dialog_items .wpfa-error .fa, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-error .fas, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-error .fab, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-error .far, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-error .fal, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-error, 
                #wpfa_dialog #wpfa_dialog_items .wpfa-item-col-error{ color: '. $WPFCOLOR_42 .'; }
                #wpfa_dialog .wpfa-button{ background-color: '. $WPFCOLOR_12 .'; border-color: '. $WPFCOLOR_14 .'; }
                #wpfa_dialog .wpfa-button.wpfa-button-add-to-post{ background-color: '. $WPFCOLOR_32 .'; border-color: '. $WPFCOLOR_33 .'; }
                #wpfa_dialog .wpfa-button.wpfa-button-cancel{ background-color: '. $WPFCOLOR_5 .'; border-color: '. $WPFCOLOR_6 .'; }
                #wpfa_dialog .wpfa-button.wpfa-button-delete{ background-color: '. $WPFCOLOR_41 .'; border-color: '. $WPFCOLOR_42 .'; }
            ';
            return $css;
        }

    }

    if( !function_exists('WPF_ATTACH') ){
        function WPF_ATTACH(){
            return wpForoAttachments::instance();
        }
    }

    require_once(WPFOROATTACH_DIR . "/includes/functions.php");
}